/* advautoage_h_gm2d_point_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
        Points
*************************/

/* Point */

#define MAX_POINTS 100000

int NPoints;
double Point_coord[MAX_POINTS][2];



void ReadPointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  assert(NPoints == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPoints);
  assert(0 <= NPoints);
  assert(NPoints <= MAX_POINTS);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    int pointId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &pointId,
	    &x, &y);
    assert(pointId == iPoint);

    Point_coord[iPoint][0] = x;
    Point_coord[iPoint][1] = y; 
  }

  fclose (fp);
}

void Point_getRange
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iPoint;

  assert(0 < NPoints);
  
  *minRangeX_OUT = *maxRangeX_OUT = Point_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Point_coord[0][1];
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
  }
}



/********************************
	 Visualization
********************************/

void SetView (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  
  Point_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 
}

void RedrawView (void) 
{
  int iPoint;

  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1];

    AutoGL_SetColor (1, 1, 1);
    AutoGL_DrawCrossMark2D (x, y, 0.0,
			    3);
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm2d_point_view  point_file \n");
    exit (1);
  }

  ReadPointFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
