/* advautoage_h_gm2d_polygon_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/********************************
         Polygon Data
********************************/

/* Polygon */

#define MAX_POLYGONS 1000
#define MAX_POLYGON_LOOPS 100
#define MAX_POLYGON_VERTEXS 100

int NPolygons;
int Polygon_nLoops[MAX_POLYGONS];
int Polygon_nVertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS];
double Polygon_vertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS][MAX_POLYGON_VERTEXS][2];



void ReadPolygonFile (const char *fileName)
{
  FILE *fp;
  int iPolygon;
  int iLoop;
  int iVertex;

  assert(NPolygons == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPolygons);
  assert(0 <= NPolygons);
  assert(NPolygons <= MAX_POLYGONS);
  
  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int polygonId;
    int nLoops;
    
    fscanf (fp, "%d %d", 
	    &polygonId,
	    &nLoops);
    assert(polygonId == iPolygon);
    assert(0 < nLoops);
    
    Polygon_nLoops[iPolygon] = nLoops;
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs;

      fscanf (fp, "%d", 
	      &nVertexs);
      assert(2 < nVertexs);
      
      Polygon_nVertexs[iPolygon][iLoop] = nVertexs;

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x, y;

	fscanf (fp, "%lf %lf", 
		&x, &y);
	
	Polygon_vertexs[iPolygon][iLoop][iVertex][0] = x;
	Polygon_vertexs[iPolygon][iLoop][iVertex][1] = y; 
      }
    }
  }

  fclose (fp);
}

void Polygon_getRange
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iPolygon;
  int iLoop;
  int iVertex;
  
  assert(0 < NPolygons);
  
  *minRangeX_OUT = *maxRangeX_OUT = Polygon_vertexs[0][0][0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Polygon_vertexs[0][0][0][1];
  
  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];
      
      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	
	if (x < *minRangeX_OUT) *minRangeX_OUT = x;
	if (y < *minRangeY_OUT) *minRangeY_OUT = y;
	
	if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
	if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
      }
    }
  }
}



/********************************
	 Visualization
********************************/

int Vertex = 1;
int Edge = 1;
double Ratio = 0.9;

void SetView (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  
  Polygon_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 
}

void DrawPolygon (void) 
{
  int iPolygon;
  int iLoop;
  int iVertex;

  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];

    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];
      double centerX, centerY;
      
      centerX = 0.0;
      centerY = 0.0;
      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	
	centerX += x;
	centerY += y;
      }
      centerX /= nVertexs;
      centerY /= nVertexs;

      if (Vertex) {
	AutoGL_SetColor (1, 0, 0);
	
	for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	  double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	  double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];

	  double vx = centerX + (x - centerX) * Ratio;
	  double vy = centerY + (y - centerY) * Ratio;

	  AutoGL_DrawCrossMark2D (vx, vy, 0.0,
				  3);
	}
      }

      if (Edge) {
	AutoGL_SetColor (0, 1, 0);
	
	for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	  int v0 = iVertex;
	  int v1 = (iVertex + 1) % nVertexs;

	  double x0 = Polygon_vertexs[iPolygon][iLoop][v0][0];
	  double y0 = Polygon_vertexs[iPolygon][iLoop][v0][1];

	  double x1 = Polygon_vertexs[iPolygon][iLoop][v1][0];
	  double y1 = Polygon_vertexs[iPolygon][iLoop][v1][1];

	  double vx0 = centerX + (x0 - centerX) * Ratio;
	  double vy0 = centerY + (y0 - centerY) * Ratio;
	  
	  double vx1 = centerX + (x1 - centerX) * Ratio;
	  double vy1 = centerY + (y1 - centerY) * Ratio;
	  
	  AutoGL_DrawLine (vx0, vy0, 0.0,
			   vx1, vy1, 0.0);
	}
      }
    }
  }
}

void RedrawView (void) 
{
  DrawPolygon ();
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm2d_polygon_view  polygon_file  \n");
    exit (1);
  }

  ReadPolygonFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");
  
  AutoGL_AddBoolean (&Vertex, "Vertex");
  AutoGL_AddBoolean (&Edge, "Edge");
  AutoGL_AddReal (&Ratio, "Ratio");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
