/* advautoage_h_gm2d_seg_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
        Segments
*************************/

/* Segment */

#define MAX_SEGS 100000

int NSegs;
double Seg_coord0[MAX_SEGS][2];
double Seg_coord1[MAX_SEGS][2];



void ReadSegFile (const char *fileName)
{
  FILE *fp;
  int iSeg;

  assert(NSegs == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NSegs);
  assert(0 <= NSegs);
  assert(NSegs <= MAX_SEGS);
  
  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    int segId;
    double x0, y0, x1, y1;

    fscanf (fp, "%d %lf %lf %lf %lf", 
	    &segId,
	    &x0, &y0, 
	    &x1, &y1);
    assert(segId == iSeg);

    Seg_coord0[iSeg][0] = x0;
    Seg_coord0[iSeg][1] = y0; 

    Seg_coord1[iSeg][0] = x1;
    Seg_coord1[iSeg][1] = y1; 
  }

  fclose (fp);
}

void Seg_getRange
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iSeg;

  assert(0 < NSegs);
  
  *minRangeX_OUT = *maxRangeX_OUT = Seg_coord0[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Seg_coord0[0][1];
  
  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double x0 = Seg_coord0[iSeg][0];
    double y0 = Seg_coord0[iSeg][1];
    
    double x1 = Seg_coord1[iSeg][0];
    double y1 = Seg_coord1[iSeg][1];
    
    if (x0 < *minRangeX_OUT) *minRangeX_OUT = x0;
    if (y0 < *minRangeY_OUT) *minRangeY_OUT = y0;

    if (*maxRangeX_OUT < x0) *maxRangeX_OUT = x0;
    if (*maxRangeY_OUT < y0) *maxRangeY_OUT = y0;

    if (x1 < *minRangeX_OUT) *minRangeX_OUT = x1;
    if (y1 < *minRangeY_OUT) *minRangeY_OUT = y1;

    if (*maxRangeX_OUT < x1) *maxRangeX_OUT = x1;
    if (*maxRangeY_OUT < y1) *maxRangeY_OUT = y1;
  }
}



/********************************
	 Visualization
********************************/

int Fill = 1;
int Vertex = 1;
double Ratio = 0.9;

void SetView (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  
  Seg_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 
}

void RedrawView (void) 
{
  int iSeg;

  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double x0 = Seg_coord0[iSeg][0];
    double y0 = Seg_coord0[iSeg][1];
    
    double x1 = Seg_coord1[iSeg][0];
    double y1 = Seg_coord1[iSeg][1];

    double centerX = (x0 + x1) / 2;
    double centerY = (y0 + y1) / 2;
    
    double vx0 = centerX + (x0 - centerX) * Ratio;
    double vx1 = centerX + (x1 - centerX) * Ratio;

    double vy0 = centerY + (y0 - centerY) * Ratio;
    double vy1 = centerY + (y1 - centerY) * Ratio;

    if (Vertex) {
      AutoGL_SetColor (1, 0, 0);
      AutoGL_DrawCrossMark2D (vx0, vy0, 0.0,
			      3);
      AutoGL_DrawCrossMark2D (vx1, vy1, 0.0,
			      3);
    }

    if (Fill) {
      AutoGL_SetColor (0, 1, 1);
      AutoGL_DrawLine (vx0, vy0, 0.0,
		       vx1, vy1, 0.0);
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm2d_seg_view  seg_file \n");
    exit (1);
  }

  ReadSegFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Fill, "Fill");
  AutoGL_AddBoolean (&Vertex, "Vertex");
  AutoGL_AddReal (&Ratio, "Ratio");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
