/* gm2d_rect.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
      Geometry Data
*************************/

/* Point */

#define MAX_POINTS 100000

int NPoints;
double Point_coord[MAX_POINTS][2];



/* Segment */

#define MAX_SEGS 100000

int NSegs;
double Seg_coord0[MAX_SEGS][2];
double Seg_coord1[MAX_SEGS][2];



int AddPoint 
(double x, double y)
{
  int pointId = NPoints;

  assert(NPoints < MAX_POINTS);
  NPoints++;
  
  Point_coord[pointId][0] = x;
  Point_coord[pointId][1] = y;

  return pointId;
}

int AddSeg 
(double x0, double y0,
 double x1, double y1)
{
  int segId = NSegs;

  assert(NSegs < MAX_SEGS);
  NSegs++;
  
  Seg_coord0[segId][0] = x0;
  Seg_coord0[segId][1] = y0;

  Seg_coord1[segId][0] = x1;
  Seg_coord1[segId][1] = y1;

  return segId;
}

int AddSegByPoint 
(int pointId0, int pointId1)
{
  assert(0 <= pointId0);
  assert(pointId0 < NPoints);
  assert(0 <= pointId1);
  assert(pointId1 < NPoints);

  return AddSeg (Point_coord[pointId0][0],
		 Point_coord[pointId0][1],
		 
		 Point_coord[pointId1][0],
		 Point_coord[pointId1][1]);
}

void WritePointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", NPoints);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1]; 

    fprintf (fp, "%d   %22.15e %22.15e\n", 
	     iPoint,
	     x, y);
  }

  fclose (fp);
}

void WriteSegFile (const char *fileName)
{
  FILE *fp;
  int iSeg;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", NSegs);
  
  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double x0 = Seg_coord0[iSeg][0];
    double y0 = Seg_coord0[iSeg][1]; 

    double x1 = Seg_coord1[iSeg][0];
    double y1 = Seg_coord1[iSeg][1]; 

    fprintf (fp, "%d   %22.15e %22.15e   %22.15e %22.15e\n", 
	     iSeg,
	     x0, y0, 
	     x1, y1);
  }

  fclose (fp);
}



/************************
      Command
*************************/

void PerformCommand 
(double xSize, double ySize)
{
  double x0 = 0.0;
  double y0 = 0.0;
  double x1 = xSize;
  double y1 = ySize;

  AddPoint (x0, y0);
  AddPoint (x1, y0);
  AddPoint (x1, y1);
  AddPoint (x0, y1);

  AddSegByPoint (0, 1);
  AddSegByPoint (1, 2);
  AddSegByPoint (2, 3);
  AddSegByPoint (3, 0);
}



int main (int argc, char *argv[])
{
  double xSize, ySize;

  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " gm2d_rect  x y "
       "   point_file_OUT  seg_file_OUT \n");
    exit (1);
  }

  xSize = atof (argv[1]);
  ySize = atof (argv[2]);
  PerformCommand (xSize, ySize);

  WritePointFile (argv[3]);
  WriteSegFile (argv[4]);

  return 0;
}
