/* advautoage_h_gm3d_face_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/********************************
         Face Data
********************************/

/* Point */

#define MAX_POINTS 10000

int NPoints;
double Point_coord[MAX_POINTS][3];



/* Edge */

#define MAX_EDGES 10000

int NEdges;
int Edge_pointId0[MAX_EDGES];
int Edge_pointId1[MAX_EDGES];



/* Face */

#define MAX_FACES 3000
#define MAX_FACE_LOOPS 100
#define MAX_FACE_VERTEXS 100

int NFaces;
int Face_nLoops[MAX_FACES];
int Face_nVertexs[MAX_FACES][MAX_FACE_LOOPS];
int Face_pointIds[MAX_FACES][MAX_FACE_LOOPS][MAX_FACE_VERTEXS];
int Face_edgeIds[MAX_FACES][MAX_FACE_LOOPS][MAX_FACE_VERTEXS];



void ReadPointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  assert(NPoints == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPoints);
  assert(0 <= NPoints);
  assert(NPoints <= MAX_POINTS);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    int pointId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &pointId,
	    &x, &y, &z);
    assert(pointId == iPoint);

    Point_coord[iPoint][0] = x;
    Point_coord[iPoint][1] = y; 
    Point_coord[iPoint][2] = z; 
  }

  fclose (fp);
}

void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(0 < NPoints);
  assert(NEdges == 0);
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int pointId0, pointId1;

    fscanf (fp, "%d %d %d", 
	    &edgeId,
	    &pointId0, &pointId1);
    assert(edgeId == iEdge);
    assert(0 <= pointId0);
    assert(pointId0 < NPoints);
    assert(0 <= pointId1);
    assert(pointId1 < NPoints);

    Edge_pointId0[iEdge] = pointId0;
    Edge_pointId1[iEdge] = pointId1;
  }

  fclose (fp);
}

void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  int iLoop;
  int iVertex;

  assert(0 < NPoints);
  assert(0 < NEdges);
  assert(NFaces == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int faceId;
    int nLoops;
    
    fscanf (fp, "%d %d", 
	    &faceId,
	    &nLoops);
    assert(faceId == iFace);
    assert(0 < nLoops);
    
    Face_nLoops[iFace] = nLoops;
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs;

      fscanf (fp, "%d", 
	      &nVertexs);
      assert(2 < nVertexs);
      
      Face_nVertexs[iFace][iLoop] = nVertexs;
      
      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int pointId;
	
	fscanf (fp, "%d", 
		&pointId);
	assert(0 <= pointId);
	assert(pointId < NPoints);
	
	Face_pointIds[iFace][iLoop][iVertex] = pointId;
      }

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int edgeId;

	fscanf (fp, "%d", 
		&edgeId);
	assert(0 <= edgeId);
	assert(edgeId < NEdges);
	
	Face_edgeIds[iFace][iLoop][iVertex] = edgeId;
      }

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int v0 = iVertex;
	int v1 = (iVertex + 1) % nVertexs;

	int p0 = Face_pointIds[iFace][iLoop][v0];
	int p1 = Face_pointIds[iFace][iLoop][v1];

	int edgeId = Face_edgeIds[iFace][iLoop][iVertex];
	int ep0 = Edge_pointId0[edgeId];
	int ep1 = Edge_pointId1[edgeId];

	assert((p0 == ep0 && p1 == ep1)
	       || (p0 == ep1 && p1 == ep0));
      }
    }
  }

  fclose (fp);
}

void Point_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iPoint;

  assert(0 < NPoints);
  
  *minRangeX_OUT = *maxRangeX_OUT = Point_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Point_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Point_coord[0][2];
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1];
    double z = Point_coord[iPoint][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}



/********************************
	 Visualization
********************************/

int Vertex = 1;
int Edge = 1;
int Face = 1;
double Ratio = 0.9;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Point_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void DrawVertex (void) 
{
  int iPoint;
  
  AutoGL_SetColor (1, 0, 0);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1];
    double z = Point_coord[iPoint][2];
    
    AutoGL_DrawCrossMark3D (x, y, z,
			    3);
  }
}

void DrawEdge (void) 
{
  int iEdge;
  
  AutoGL_SetColor (0, 1, 0);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int pointId0 = Edge_pointId0[iEdge];
    int pointId1 = Edge_pointId1[iEdge];
    
    double x0 = Point_coord[pointId0][0];
    double y0 = Point_coord[pointId0][1];
    double z0 = Point_coord[pointId0][2];
    
    double x1 = Point_coord[pointId1][0];
    double y1 = Point_coord[pointId1][1];
    double z1 = Point_coord[pointId1][2];
    
    double centerX = (x0 + x1) / 2;
    double centerY = (y0 + y1) / 2;
    double centerZ = (z0 + z1) / 2;
    
    double vx0 = centerX + (x0 - centerX) * Ratio;
    double vy0 = centerY + (y0 - centerY) * Ratio;
    double vz0 = centerZ + (z0 - centerZ) * Ratio;
    
    double vx1 = centerX + (x1 - centerX) * Ratio;
    double vy1 = centerY + (y1 - centerY) * Ratio;
    double vz1 = centerZ + (z1 - centerZ) * Ratio;
    
    AutoGL_DrawLine (vx0, vy0, vz0,
		     vx1, vy1, vz1);
  }
}

void DrawFace (void) 
{
  int iFace;
  int iLoop;
  int iVertex;

  for (iFace = 0; iFace < NFaces; iFace++) {
    int nLoops = Face_nLoops[iFace];

    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Face_nVertexs[iFace][iLoop];
      double centerX, centerY, centerZ;
      
      centerX = 0.0;
      centerY = 0.0;
      centerZ = 0.0;
      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int pointId = Face_pointIds[iFace][iLoop][iVertex];

	double x = Point_coord[pointId][0];
	double y = Point_coord[pointId][1];
	double z = Point_coord[pointId][2];
	
	centerX += x;
	centerY += y;
	centerZ += z;
      }
      centerX /= nVertexs;
      centerY /= nVertexs;
      centerZ /= nVertexs;

      if (iLoop == 0) {
	AutoGL_SetColor (1, 0, 0);
      } else {
	AutoGL_SetColor (1, 0, 1);
      }
	
      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int edgeId = Face_edgeIds[iFace][iLoop][iVertex];
	int pointId0 = Edge_pointId0[edgeId];
	int pointId1 = Edge_pointId1[edgeId];

	double x0 = Point_coord[pointId0][0];
	double y0 = Point_coord[pointId0][1];
	double z0 = Point_coord[pointId0][2];

	double x1 = Point_coord[pointId1][0];
	double y1 = Point_coord[pointId1][1];
	double z1 = Point_coord[pointId1][2];
	
	double vx0 = centerX + (x0 - centerX) * Ratio;
	double vy0 = centerY + (y0 - centerY) * Ratio;
	double vz0 = centerZ + (z0 - centerZ) * Ratio;
	
	double vx1 = centerX + (x1 - centerX) * Ratio;
	double vy1 = centerY + (y1 - centerY) * Ratio;
	double vz1 = centerZ + (z1 - centerZ) * Ratio;
	
	AutoGL_DrawLine (vx0, vy0, vz0,
			 vx1, vy1, vz1);
      }
    }
  }
}

void RedrawView (void) 
{
  AutoGL_DrawCoordinateSystem3D (20);

  if (Vertex) {
    DrawVertex ();
  }
  if (Edge) {
    DrawEdge ();
  }
  if (Face) {
    DrawFace ();
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm3d_face_view  \n"
       "        point_file  \n"
       "        edge_file  face_file  \n");
    exit (1);
  }

  ReadPointFile (argv[1]);
  ReadEdgeFile (argv[2]);
  ReadFaceFile (argv[3]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");
  
  AutoGL_AddBoolean (&Vertex, "Vertex");
  AutoGL_AddBoolean (&Edge, "Edge");
  AutoGL_AddBoolean (&Face, "Face");
  AutoGL_AddReal (&Ratio, "Ratio");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
