/* advautoage_h_gm3d_point_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
        Points
*************************/

/* Point */

#define MAX_POINTS 100000

int NPoints;
double Point_coord[MAX_POINTS][3];



void ReadPointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  assert(NPoints == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPoints);
  assert(0 <= NPoints);
  assert(NPoints <= MAX_POINTS);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    int pointId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &pointId,
	    &x, &y, &z);
    assert(pointId == iPoint);

    Point_coord[iPoint][0] = x;
    Point_coord[iPoint][1] = y; 
    Point_coord[iPoint][2] = z; 
  }

  fclose (fp);
}

void Point_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iPoint;

  assert(0 < NPoints);
  
  *minRangeX_OUT = *maxRangeX_OUT = Point_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Point_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Point_coord[0][2];
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1];
    double z = Point_coord[iPoint][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}



/********************************
	 Visualization
********************************/

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Point_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void RedrawView (void) 
{
  int iPoint;

  AutoGL_DrawCoordinateSystem3D (20);

  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1];
    double z = Point_coord[iPoint][2];

    AutoGL_SetColor (1, 1, 1);
    AutoGL_DrawCrossMark3D (x, y, z,
			    3);
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm3d_point_view  point_file \n");
    exit (1);
  }

  ReadPointFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
