/* advautoage_h_gm3d_seg_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
        Segments
*************************/

/* Segment */

#define MAX_SEGS 100000

int NSegs;
double Seg_coord0[MAX_SEGS][3];
double Seg_coord1[MAX_SEGS][3];



void ReadSegFile (const char *fileName)
{
  FILE *fp;
  int iSeg;

  assert(NSegs == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NSegs);
  assert(0 < NSegs);
  assert(NSegs <= MAX_SEGS);
  
  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    int segId;
    double x0, y0, z0, x1, y1, z1;

    fscanf (fp, "%d %lf %lf %lf %lf %lf %lf", 
	    &segId,
	    &x0, &y0, &z0, 
	    &x1, &y1, &z1);
    assert(segId == iSeg);

    Seg_coord0[iSeg][0] = x0;
    Seg_coord0[iSeg][1] = y0; 
    Seg_coord0[iSeg][2] = z0; 

    Seg_coord1[iSeg][0] = x1;
    Seg_coord1[iSeg][1] = y1; 
    Seg_coord1[iSeg][2] = z1; 
  }

  fclose (fp);
}

void Seg_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iSeg;

  assert(0 < NSegs);
  
  *minRangeX_OUT = *maxRangeX_OUT = Seg_coord0[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Seg_coord0[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Seg_coord0[0][2];
  
  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double x0 = Seg_coord0[iSeg][0];
    double y0 = Seg_coord0[iSeg][1];
    double z0 = Seg_coord0[iSeg][2];
    
    double x1 = Seg_coord1[iSeg][0];
    double y1 = Seg_coord1[iSeg][1];
    double z1 = Seg_coord1[iSeg][2];
    
    if (x0 < *minRangeX_OUT) *minRangeX_OUT = x0;
    if (y0 < *minRangeY_OUT) *minRangeY_OUT = y0;
    if (z0 < *minRangeZ_OUT) *minRangeZ_OUT = z0;

    if (*maxRangeX_OUT < x0) *maxRangeX_OUT = x0;
    if (*maxRangeY_OUT < y0) *maxRangeY_OUT = y0;
    if (*maxRangeZ_OUT < z0) *maxRangeZ_OUT = z0;

    if (x1 < *minRangeX_OUT) *minRangeX_OUT = x1;
    if (y1 < *minRangeY_OUT) *minRangeY_OUT = y1;
    if (z1 < *minRangeZ_OUT) *minRangeZ_OUT = z1;

    if (*maxRangeX_OUT < x1) *maxRangeX_OUT = x1;
    if (*maxRangeY_OUT < y1) *maxRangeY_OUT = y1;
    if (*maxRangeZ_OUT < z1) *maxRangeZ_OUT = z1;
  }
}



/********************************
	 Visualization
********************************/

int Fill = 1;
int Vertex = 1;
double Ratio = 0.9;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Seg_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void RedrawView (void) 
{
  int iSeg;

  AutoGL_DrawCoordinateSystem3D (20);

  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double x0 = Seg_coord0[iSeg][0];
    double y0 = Seg_coord0[iSeg][1];
    double z0 = Seg_coord0[iSeg][2];
    
    double x1 = Seg_coord1[iSeg][0];
    double y1 = Seg_coord1[iSeg][1];
    double z1 = Seg_coord1[iSeg][2];

    double centerX = (x0 + x1) / 2;
    double centerY = (y0 + y1) / 2;
    double centerZ = (z0 + z1) / 2;
    
    double vx0 = centerX + (x0 - centerX) * Ratio;
    double vx1 = centerX + (x1 - centerX) * Ratio;

    double vy0 = centerY + (y0 - centerY) * Ratio;
    double vy1 = centerY + (y1 - centerY) * Ratio;

    double vz0 = centerZ + (z0 - centerZ) * Ratio;
    double vz1 = centerZ + (z1 - centerZ) * Ratio;

    if (Vertex) {
      AutoGL_SetColor (1, 0, 0);
      AutoGL_DrawCrossMark3D (vx0, vy0, vz0,
			      3);
      AutoGL_DrawCrossMark3D (vx1, vy1, vz1,
			      3);
    }

    if (Fill) {
      AutoGL_SetColor (0, 1, 1);
      AutoGL_DrawLine (vx0, vy0, vz0,
		       vx1, vy1, vz1);
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm3d_seg_view  seg_file \n");
    exit (1);
  }

  ReadSegFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Fill, "Fill");
  AutoGL_AddBoolean (&Vertex, "Vertex");
  AutoGL_AddReal (&Ratio, "Ratio");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
