/* advautoage_h_gm3d_tri_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
        Triangles
*************************/

/* Triangle */

#define MAX_TRIS 100000

int NTris;
double Tri_coord0[MAX_TRIS][3];
double Tri_coord1[MAX_TRIS][3];
double Tri_coord2[MAX_TRIS][3];



void ReadTriFile (const char *fileName)
{
  FILE *fp;
  int iTri;

  assert(NTris == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NTris);
  assert(0 <= NTris);
  assert(NTris <= MAX_TRIS);
  
  for (iTri = 0; iTri < NTris; iTri++) {
    int triId;
    double x0, y0, z0, x1, y1, z1, x2, y2, z2;

    fscanf (fp, "%d %lf %lf %lf %lf %lf %lf %lf %lf %lf", 
	    &triId,
	    &x0, &y0, &z0, 
	    &x1, &y1, &z1, 
	    &x2, &y2, &z2);
    assert(triId == iTri);

    Tri_coord0[iTri][0] = x0;
    Tri_coord0[iTri][1] = y0; 
    Tri_coord0[iTri][2] = z0; 

    Tri_coord1[iTri][0] = x1;
    Tri_coord1[iTri][1] = y1; 
    Tri_coord1[iTri][2] = z1; 

    Tri_coord2[iTri][0] = x2;
    Tri_coord2[iTri][1] = y2; 
    Tri_coord2[iTri][2] = z2; 
  }

  fclose (fp);
}

void Tri_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iTri;

  assert(0 < NTris);
  
  *minRangeX_OUT = *maxRangeX_OUT = Tri_coord0[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Tri_coord0[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Tri_coord0[0][2];
  
  for (iTri = 0; iTri < NTris; iTri++) {
    double x0 = Tri_coord0[iTri][0];
    double y0 = Tri_coord0[iTri][1];
    double z0 = Tri_coord0[iTri][2];
    
    double x1 = Tri_coord1[iTri][0];
    double y1 = Tri_coord1[iTri][1];
    double z1 = Tri_coord1[iTri][2];
    
    double x2 = Tri_coord2[iTri][0];
    double y2 = Tri_coord2[iTri][1];
    double z2 = Tri_coord2[iTri][2];
    
    if (x0 < *minRangeX_OUT) *minRangeX_OUT = x0;
    if (y0 < *minRangeY_OUT) *minRangeY_OUT = y0;
    if (z0 < *minRangeZ_OUT) *minRangeZ_OUT = z0;

    if (*maxRangeX_OUT < x0) *maxRangeX_OUT = x0;
    if (*maxRangeY_OUT < y0) *maxRangeY_OUT = y0;
    if (*maxRangeZ_OUT < z0) *maxRangeZ_OUT = z0;

    if (x1 < *minRangeX_OUT) *minRangeX_OUT = x1;
    if (y1 < *minRangeY_OUT) *minRangeY_OUT = y1;
    if (z1 < *minRangeZ_OUT) *minRangeZ_OUT = z1;

    if (*maxRangeX_OUT < x1) *maxRangeX_OUT = x1;
    if (*maxRangeY_OUT < y1) *maxRangeY_OUT = y1;
    if (*maxRangeZ_OUT < z1) *maxRangeZ_OUT = z1;

    if (x2 < *minRangeX_OUT) *minRangeX_OUT = x2;
    if (y2 < *minRangeY_OUT) *minRangeY_OUT = y2;
    if (z2 < *minRangeZ_OUT) *minRangeZ_OUT = z2;

    if (*maxRangeX_OUT < x2) *maxRangeX_OUT = x2;
    if (*maxRangeY_OUT < y2) *maxRangeY_OUT = y2;
    if (*maxRangeZ_OUT < z2) *maxRangeZ_OUT = z2;
  }
}



/********************************
	 Visualization
********************************/

int Fill = 1;
int Edge = 1;
double Ratio = 0.9;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Tri_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void RedrawView (void) 
{
  int iTri;

  AutoGL_DrawCoordinateSystem3D (20);

  for (iTri = 0; iTri < NTris; iTri++) {
    double x0 = Tri_coord0[iTri][0];
    double y0 = Tri_coord0[iTri][1];
    double z0 = Tri_coord0[iTri][2];
    
    double x1 = Tri_coord1[iTri][0];
    double y1 = Tri_coord1[iTri][1];
    double z1 = Tri_coord1[iTri][2];
    
    double x2 = Tri_coord2[iTri][0];
    double y2 = Tri_coord2[iTri][1];
    double z2 = Tri_coord2[iTri][2];

    double centerX = (x0 + x1 + x2) / 3;
    double centerY = (y0 + y1 + y2) / 3;
    double centerZ = (z0 + z1 + z2) / 3;
    
    double vx0 = centerX + (x0 - centerX) * Ratio;
    double vx1 = centerX + (x1 - centerX) * Ratio;
    double vx2 = centerX + (x2 - centerX) * Ratio;

    double vy0 = centerY + (y0 - centerY) * Ratio;
    double vy1 = centerY + (y1 - centerY) * Ratio;
    double vy2 = centerY + (y2 - centerY) * Ratio;

    double vz0 = centerZ + (z0 - centerZ) * Ratio;
    double vz1 = centerZ + (z1 - centerZ) * Ratio;
    double vz2 = centerZ + (z2 - centerZ) * Ratio;

    if (Edge) {
      AutoGL_SetColor (1, 0, 0);
      AutoGL_DrawLine (vx0, vy0, vz0,
		       vx1, vy1, vz1);
      AutoGL_DrawLine (vx1, vy1, vz1,
		       vx2, vy2, vz2);
      AutoGL_DrawLine (vx2, vy2, vz2,
		       vx0, vy0, vz0);
    }

    if (Fill) {
      AutoGL_SetColor (0, 1, 1);
      AutoGL_DrawTriangle (vx0, vy0, vz0,
			   vx1, vy1, vz1,
			   vx2, vy2, vz2);
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm3d_tri_view  tri_file \n");
    exit (1);
  }

  ReadTriFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Fill, "Fill");
  AutoGL_AddBoolean (&Edge, "Edge");
  AutoGL_AddReal (&Ratio, "Ratio");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
