/* advautoage_h_gm3d_negate_polygon.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/********************************
         Polygon Data
********************************/

/* Polygon */

#define MAX_POLYGONS 3000
#define MAX_POLYGON_LOOPS 100
#define MAX_POLYGON_VERTEXS 100

int NPolygons;
int Polygon_nLoops[MAX_POLYGONS];
int Polygon_nVertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS];
double Polygon_vertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS][MAX_POLYGON_VERTEXS][3];



void ReadPolygonFile (const char *fileName)
{
  FILE *fp;
  int iPolygon;
  int iLoop;
  int iVertex;

  assert(NPolygons == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPolygons);
  assert(0 < NPolygons);
  assert(NPolygons <= MAX_POLYGONS);
  
  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int polygonId;
    int nLoops;
    
    fscanf (fp, "%d %d", 
	    &polygonId,
	    &nLoops);
    assert(polygonId == iPolygon);
    assert(0 < nLoops);
    
    Polygon_nLoops[iPolygon] = nLoops;
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs;

      fscanf (fp, "%d", 
	      &nVertexs);
      assert(2 < nVertexs);
      
      Polygon_nVertexs[iPolygon][iLoop] = nVertexs;

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x, y, z;

	fscanf (fp, "%lf %lf %lf", 
		&x, &y, &z);
	
	Polygon_vertexs[iPolygon][iLoop][iVertex][0] = x;
	Polygon_vertexs[iPolygon][iLoop][iVertex][1] = y; 
	Polygon_vertexs[iPolygon][iLoop][iVertex][2] = z; 
      }
    }
  }

  fclose (fp);
}

void WritePolygonFile (const char *fileName)
{
  FILE *fp;
  int iPolygon;
  int iLoop;
  int iVertex;

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NPolygons);

  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];
    
    fprintf (fp, "%d   %d\n",
	     iPolygon,
	     nLoops);

    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];
      
      fprintf (fp, "  %d\n",
	       nVertexs);

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	double z = Polygon_vertexs[iPolygon][iLoop][iVertex][2];

	fprintf (fp, "  %22.15e %22.15e %22.15e\n",
		 x, y, z);
      }
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iPolygon;
  int iLoop;
  int iVertex;

  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];

      for (iVertex = 0; iVertex < nVertexs / 2; iVertex++) {
	int reverseVertexId = nVertexs - 1 - iVertex;

	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	double z = Polygon_vertexs[iPolygon][iLoop][iVertex][2];

	Polygon_vertexs[iPolygon][iLoop][iVertex][0]
	  = Polygon_vertexs[iPolygon][iLoop][reverseVertexId][0];
	Polygon_vertexs[iPolygon][iLoop][iVertex][1]
	  = Polygon_vertexs[iPolygon][iLoop][reverseVertexId][1];
	Polygon_vertexs[iPolygon][iLoop][iVertex][2]
	  = Polygon_vertexs[iPolygon][iLoop][reverseVertexId][2];

	Polygon_vertexs[iPolygon][iLoop][reverseVertexId][0] = x;
	Polygon_vertexs[iPolygon][iLoop][reverseVertexId][1] = y;
	Polygon_vertexs[iPolygon][iLoop][reverseVertexId][2] = z;
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 3) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm3d_negate_polygon  polygon_file  \n"
       "          polygon_file_OUT \n");
    exit (1);
  }

  ReadPolygonFile (argv[1]);

  PerformCommand ();

  WritePolygonFile (argv[2]);

  return 0;
}
