/* advautoage_h_gm3d_polygon2face.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define DEFAULT_EPS 0.00001

double Eps = DEFAULT_EPS;



double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/********************************
         Polygon Data
********************************/

/* Polygon */

#define MAX_POLYGONS 2500
#define MAX_POLYGON_LOOPS 100
#define MAX_POLYGON_VERTEXS 100

int NPolygons;
int Polygon_nLoops[MAX_POLYGONS];
int Polygon_nVertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS];
double Polygon_vertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS][MAX_POLYGON_VERTEXS][3];



void ReadPolygonFile (const char *fileName)
{
  FILE *fp;
  int iPolygon;
  int iLoop;
  int iVertex;

  assert(NPolygons == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPolygons);
  assert(0 < NPolygons);
  assert(NPolygons <= MAX_POLYGONS);
  
  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int polygonId;
    int nLoops;
    
    fscanf (fp, "%d %d", 
	    &polygonId,
	    &nLoops);
    assert(polygonId == iPolygon);
    assert(0 < nLoops);
    
    Polygon_nLoops[iPolygon] = nLoops;
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs;

      fscanf (fp, "%d", 
	      &nVertexs);
      assert(2 < nVertexs);
      
      Polygon_nVertexs[iPolygon][iLoop] = nVertexs;

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x, y, z;

	fscanf (fp, "%lf %lf %lf", 
		&x, &y, &z);
	
	Polygon_vertexs[iPolygon][iLoop][iVertex][0] = x;
	Polygon_vertexs[iPolygon][iLoop][iVertex][1] = y; 
	Polygon_vertexs[iPolygon][iLoop][iVertex][2] = z; 
      }
    }
  }

  fclose (fp);
}



/************************
        Face Data
*************************/

/* Point */

#define MAX_POINTS 10000

int NPoints;
double Point_coord[MAX_POINTS][3];



/* Edge */

#define MAX_EDGES 10000

int NEdges;
int Edge_pointId0[MAX_EDGES];
int Edge_pointId1[MAX_EDGES];



/* Face */

#define MAX_FACES 2500
#define MAX_FACE_LOOPS 100
#define MAX_FACE_VERTEXS 100

int NFaces;
int Face_nLoops[MAX_FACES];
int Face_nVertexs[MAX_FACES][MAX_FACE_LOOPS];
int Face_pointIds[MAX_FACES][MAX_FACE_LOOPS][MAX_FACE_VERTEXS];
int Face_edgeIds[MAX_FACES][MAX_FACE_LOOPS][MAX_FACE_VERTEXS];



int AddNewPoint (double x, double y, double z)
{
  int pointId;

  pointId = NPoints;

  assert(NPoints < MAX_POINTS);
  NPoints++;

  Point_coord[pointId][0] = x;
  Point_coord[pointId][1] = y;
  Point_coord[pointId][2] = z;

  return pointId;
}

int FindPoint (double x, double y, double z)
{
  int iPoint;

  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double px = Point_coord[iPoint][0];
    double py = Point_coord[iPoint][1];
    double pz = Point_coord[iPoint][2];

    double dist = Distance (px, py, pz, 
			    x, y, z);
    if (dist < Eps) {
      return iPoint;
    }
  }
  return -1;
}

int GetPoint (double x, double y, double z)
{
  int pointId = FindPoint (x, y, z);

  if (pointId == -1) {
    pointId = AddNewPoint (x, y, z);
  }
  assert(pointId != -1);

  return pointId;
}

int AddNewEdge (int pointId0, int pointId1)
{
  int edgeId;

  assert(0 <= pointId0);
  assert(pointId0 < MAX_POINTS);
  assert(0 <= pointId1);
  assert(pointId1 < MAX_POINTS);

  edgeId = NEdges;

  assert(NEdges < MAX_EDGES);
  NEdges++;

  Edge_pointId0[edgeId] = pointId0;
  Edge_pointId1[edgeId] = pointId1;

  return edgeId;
}

int FindEdge (int pointId0, int pointId1)
{
  int iEdge;

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int p0 = Edge_pointId0[iEdge];
    int p1 = Edge_pointId1[iEdge];

    if ((p0 == pointId0 && p1 == pointId1)
	|| (p0 == pointId1 && p1 == pointId0)) {
      return iEdge;
    }
  }
  return -1;
}

int GetEdge (int pointId0, int pointId1)
{
  int edgeId = FindEdge (pointId0, pointId1);
  
  if (edgeId == -1) {
    edgeId = AddNewEdge (pointId0, pointId1);
  }
  assert(edgeId != -1);

  return edgeId;
}

int AddNewFace (void)
{
  int faceId;

  faceId = NFaces;

  assert(NFaces < MAX_FACES);
  NFaces++;

  return faceId;
}

int AddNewLoopIntoFace (int faceId)
{
  int loopId;

  assert(0 <= faceId);
  assert(faceId < MAX_FACES);

  loopId = Face_nLoops[faceId];

  assert(Face_nLoops[faceId] < MAX_FACE_LOOPS);
  Face_nLoops[faceId]++;

  return loopId;
}

int AddNewVertexIntoFaceLoop 
(int faceId, int loopId, 
 int pointId)
{
  int vertexId;

  assert(0 <= faceId);
  assert(faceId < MAX_FACES);
  assert(0 <= loopId);
  assert(loopId < Face_nLoops[faceId]);
  assert(0 <= pointId);
  assert(pointId < MAX_POINTS);

  vertexId = Face_nVertexs[faceId][loopId];

  assert(Face_nVertexs[faceId][loopId] < MAX_FACE_VERTEXS);
  Face_nVertexs[faceId][loopId]++;

  Face_pointIds[faceId][loopId][vertexId] = pointId;

  return vertexId;
}

void SetFaceEdge (void)
{
  int iFace;
  int iLoop;
  int iVertex;

  for (iFace = 0; iFace < NFaces; iFace++) {
    int nLoops = Face_nLoops[iFace];
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Face_nVertexs[iFace][iLoop];

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int v0 = iVertex;
	int v1 = (iVertex + 1) % nVertexs;

	int pointId0 = Face_pointIds[iFace][iLoop][v0];
	int pointId1 = Face_pointIds[iFace][iLoop][v1];
	
	Face_edgeIds[iFace][iLoop][iVertex] 
	  = GetEdge (pointId0, pointId1);
      }
    }
  }
}



void WritePointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  assert(0 < NPoints);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NPoints);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	     iPoint,
	     Point_coord[iPoint][0], 
	     Point_coord[iPoint][1], 
	     Point_coord[iPoint][2]);
  }

  fclose (fp);
}

void WriteEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(0 < NEdges);
  
  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NEdges);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    fprintf (fp, "%d   %d %d\n", 
	     iEdge,
	     Edge_pointId0[iEdge],
	     Edge_pointId1[iEdge]);
  }

  fclose (fp);
}

void WriteFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  int iLoop;
  int iVertex;

  assert(0 < NFaces);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NFaces);
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int nLoops = Face_nLoops[iFace];
    
    fprintf (fp, "%d   %d\n", 
	     iFace,
	     nLoops);
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Face_nVertexs[iFace][iLoop];

      fprintf (fp, "  %d\n", 
	       nVertexs);

      fprintf (fp, "  ");
      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int pointId = Face_pointIds[iFace][iLoop][iVertex];
	
	fprintf (fp, "%d ", 
		 pointId);
      }
      fprintf (fp, "\n");

      fprintf (fp, "  ");
      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	int edgeId = Face_edgeIds[iFace][iLoop][iVertex];
	
	fprintf (fp, "%d ", 
		 edgeId);
      }
      fprintf (fp, "\n");
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iPolygon;
  int iLoop;
  int iVertex;

  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];
    int faceId = AddNewFace ();
    
    assert(faceId == iPolygon);

    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];
      int loopId = AddNewLoopIntoFace (faceId);

      assert(loopId == iLoop);

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	double z = Polygon_vertexs[iPolygon][iLoop][iVertex][2];
	int pointId = GetPoint (x, y, z);

	int vertexId = AddNewVertexIntoFaceLoop (faceId, loopId, 
						 pointId);

	assert(vertexId == iVertex);
      }
    }
  }

  SetFaceEdge ();
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm3d_polygon2face  polygon_file  \n"
       "        point_file_OUT \n"
       "        edge_file_OUT \n"
       "        face_file_OUT \n");
    exit (1);
  }

  ReadPolygonFile (argv[1]);

  PerformCommand ();

  WritePointFile (argv[2]);
  WriteEdgeFile (argv[3]);
  WriteFaceFile (argv[4]);

  return 0;
}
