/* advautoage_h_gm3d_rotate_polygon.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/********************************
         Polygon Data
********************************/

/* Polygon */

#define MAX_POLYGONS 3000
#define MAX_POLYGON_LOOPS 100
#define MAX_POLYGON_VERTEXS 100

int NPolygons;
int Polygon_nLoops[MAX_POLYGONS];
int Polygon_nVertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS];
double Polygon_vertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS][MAX_POLYGON_VERTEXS][3];



void ReadPolygonFile (const char *fileName)
{
  FILE *fp;
  int iPolygon;
  int iLoop;
  int iVertex;

  assert(NPolygons == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPolygons);
  assert(0 < NPolygons);
  assert(NPolygons <= MAX_POLYGONS);
  
  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int polygonId;
    int nLoops;
    
    fscanf (fp, "%d %d", 
	    &polygonId,
	    &nLoops);
    assert(polygonId == iPolygon);
    assert(0 < nLoops);
    
    Polygon_nLoops[iPolygon] = nLoops;
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs;

      fscanf (fp, "%d", 
	      &nVertexs);
      assert(2 < nVertexs);
      
      Polygon_nVertexs[iPolygon][iLoop] = nVertexs;

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x, y, z;

	fscanf (fp, "%lf %lf %lf", 
		&x, &y, &z);
	
	Polygon_vertexs[iPolygon][iLoop][iVertex][0] = x;
	Polygon_vertexs[iPolygon][iLoop][iVertex][1] = y; 
	Polygon_vertexs[iPolygon][iLoop][iVertex][2] = z; 
      }
    }
  }

  fclose (fp);
}

void WritePolygonFile (const char *fileName)
{
  FILE *fp;
  int iPolygon;
  int iLoop;
  int iVertex;

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NPolygons);

  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];
    
    fprintf (fp, "%d   %d\n",
	     iPolygon,
	     nLoops);

    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];
      
      fprintf (fp, "  %d\n",
	       nVertexs);

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	double z = Polygon_vertexs[iPolygon][iLoop][iVertex][2];

	fprintf (fp, "  %22.15e %22.15e %22.15e\n",
		 x, y, z);
      }
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

double CenterX;
double CenterY;
double CenterZ;

int AxisId;  /* 0 == X axis, 1 == Y axis, 2 == Z axis */
double Angle;  /* unit is degree */

void PerformCommand (void)
{
  double m[3][3];
  int iPolygon;
  int iLoop;
  int iVertex;

  {
    double s = sin (M_PI * Angle / 180);
    double c = cos (M_PI * Angle / 180);
    int i, j;
    
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
	m[i][j] = 0.0;
      }
    }
    for (i = 0; i < 3; i++) {
      m[i][i] = 1.0;
    }

    switch (AxisId) {
    case 0:  /* X axis */
      m[1][1] = c;
      m[1][2] = -s;
      m[2][1] = s;
      m[2][2] = c;
      break;
    case 1:  /* Y axis */
      m[0][0] = c;
      m[0][2] = s;
      m[2][0] = -s;
      m[2][2] = c;
      break;
    case 2:  /* Z axis */
      m[0][0] = c;
      m[0][1] = -s;
      m[1][0] = s;
      m[1][1] = c;
      break;
    default:
      assert(0);
      break;
    }
  }

  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	double z = Polygon_vertexs[iPolygon][iLoop][iVertex][2];
	
	double x2, y2, z2;
	
	x -= CenterX;
	y -= CenterY;
	z -= CenterZ;
	
	x2 = m[0][0] * x + m[0][1] * y + m[0][2] * z;
	y2 = m[1][0] * x + m[1][1] * y + m[1][2] * z;
	z2 = m[2][0] * x + m[2][1] * y + m[2][2] * z;
	
	x2 += CenterX;
	y2 += CenterY;
	z2 += CenterZ;
	
	Polygon_vertexs[iPolygon][iLoop][iVertex][0] = x2;
	Polygon_vertexs[iPolygon][iLoop][iVertex][1] = y2;
	Polygon_vertexs[iPolygon][iLoop][iVertex][2] = z2;
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 8) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautoage_h_gm3d_rotate_polygon  polygon_file  \n"
       "          centerX  centerY  centerZ \n"
       "          axisId  angle    \n"
       "          polygon_file_OUT \n"
       "\n"
       "     axisId ... 0 = X axis, 1 = Y axis, 2 = Z axis  \n"
       "     angle  ... unit is degree         \n");
    exit (1);
  }

  ReadPolygonFile (argv[1]);

  CenterX = atof (argv[2]);
  CenterY = atof (argv[3]);
  CenterZ = atof (argv[4]);

  AxisId = atoi (argv[5]);
  Angle = atof (argv[6]);

  PerformCommand ();

  WritePolygonFile (argv[7]);

  return 0;
}
