/* gm3d_gm_box.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
      Geometry Data
*************************/

/* Point */

#define MAX_POINTS 100000

int NPoints;
double Point_coord[MAX_POINTS][3];



/* Segment */

#define MAX_SEGS 100000

int NSegs;
double Seg_coord0[MAX_SEGS][3];
double Seg_coord1[MAX_SEGS][3];



/* Triangle */

#define MAX_TRIS 100000

int NTris;
double Tri_coord0[MAX_TRIS][3];
double Tri_coord1[MAX_TRIS][3];
double Tri_coord2[MAX_TRIS][3];



int AddPoint 
(double x, double y, double z)
{
  int pointId = NPoints;

  assert(NPoints < MAX_POINTS);
  NPoints++;
  
  Point_coord[pointId][0] = x;
  Point_coord[pointId][1] = y;
  Point_coord[pointId][2] = z;

  return pointId;
}

int AddSeg 
(double x0, double y0, double z0,
 double x1, double y1, double z1)
{
  int segId = NSegs;

  assert(NSegs < MAX_SEGS);
  NSegs++;
  
  Seg_coord0[segId][0] = x0;
  Seg_coord0[segId][1] = y0;
  Seg_coord0[segId][2] = z0;

  Seg_coord1[segId][0] = x1;
  Seg_coord1[segId][1] = y1;
  Seg_coord1[segId][2] = z1;

  return segId;
}

int AddTri 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2)
{
  int triId = NTris;

  assert(NTris < MAX_TRIS);
  NTris++;
  
  Tri_coord0[triId][0] = x0;
  Tri_coord0[triId][1] = y0;
  Tri_coord0[triId][2] = z0;

  Tri_coord1[triId][0] = x1;
  Tri_coord1[triId][1] = y1;
  Tri_coord1[triId][2] = z1;

  Tri_coord2[triId][0] = x2;
  Tri_coord2[triId][1] = y2;
  Tri_coord2[triId][2] = z2;

  return triId;
}

void AddQuad 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2,
 double x3, double y3, double z3)
{
  AddTri (x0, y0, z0,
	  x1, y1, z1,
	  x2, y2, z2);
  AddTri (x2, y2, z2,
	  x3, y3, z3,
	  x0, y0, z0);
}

int AddSegByPoint 
(int pointId0, int pointId1)
{
  assert(0 <= pointId0);
  assert(pointId0 < NPoints);
  assert(0 <= pointId1);
  assert(pointId1 < NPoints);

  return AddSeg (Point_coord[pointId0][0],
		 Point_coord[pointId0][1],
		 Point_coord[pointId0][2],
		 
		 Point_coord[pointId1][0],
		 Point_coord[pointId1][1],
		 Point_coord[pointId1][2]);
}

int AddTriByPoint 
(int pointId0, int pointId1, int pointId2)
{
  assert(0 <= pointId0);
  assert(pointId0 < NPoints);
  assert(0 <= pointId1);
  assert(pointId1 < NPoints);
  assert(0 <= pointId2);
  assert(pointId2 < NPoints);

  return AddTri (Point_coord[pointId0][0],
		 Point_coord[pointId0][1],
		 Point_coord[pointId0][2],
		 
		 Point_coord[pointId1][0],
		 Point_coord[pointId1][1],
		 Point_coord[pointId1][2],
		 
		 Point_coord[pointId2][0],
		 Point_coord[pointId2][1],
		 Point_coord[pointId2][2]);
}

void WritePointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", NPoints);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double x = Point_coord[iPoint][0];
    double y = Point_coord[iPoint][1]; 
    double z = Point_coord[iPoint][2]; 

    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	     iPoint,
	     x, y, z);
  }

  fclose (fp);
}

void WriteSegFile (const char *fileName)
{
  FILE *fp;
  int iSeg;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", NSegs);
  
  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double x0 = Seg_coord0[iSeg][0];
    double y0 = Seg_coord0[iSeg][1]; 
    double z0 = Seg_coord0[iSeg][2]; 

    double x1 = Seg_coord1[iSeg][0];
    double y1 = Seg_coord1[iSeg][1]; 
    double z1 = Seg_coord1[iSeg][2]; 

    fprintf (fp, "%d   %22.15e %22.15e %22.15e   %22.15e %22.15e %22.15e\n", 
	     iSeg,
	     x0, y0, z0, 
	     x1, y1, z1);
  }

  fclose (fp);
}

void WriteTriFile (const char *fileName)
{
  FILE *fp;
  int iTri;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", NTris);
  
  for (iTri = 0; iTri < NTris; iTri++) {
    double x0 = Tri_coord0[iTri][0];
    double y0 = Tri_coord0[iTri][1]; 
    double z0 = Tri_coord0[iTri][2]; 
    
    double x1 = Tri_coord1[iTri][0];
    double y1 = Tri_coord1[iTri][1]; 
    double z1 = Tri_coord1[iTri][2]; 
    
    double x2 = Tri_coord2[iTri][0];
    double y2 = Tri_coord2[iTri][1]; 
    double z2 = Tri_coord2[iTri][2]; 
    
    fprintf (fp, "%d   "
	     "%22.15e %22.15e %22.15e   "
	     "%22.15e %22.15e %22.15e   "
	     "%22.15e %22.15e %22.15e\n", 
	     iTri,
	     x0, y0, z0, 
	     x1, y1, z1, 
	     x2, y2, z2);
  }

  fclose (fp);
}



/************************
      Command
*************************/

void PerformCommand 
(double xSize, double ySize, double zSize)
{
  double x0 = 0.0;
  double y0 = 0.0;
  double z0 = 0.0;
  double x1 = xSize;
  double y1 = ySize;
  double z1 = zSize;

  AddPoint (x0, y0, z0);
  AddPoint (x1, y0, z0);
  AddPoint (x1, y1, z0);
  AddPoint (x0, y1, z0);

  AddPoint (x0, y0, z1);
  AddPoint (x1, y0, z1);
  AddPoint (x1, y1, z1);
  AddPoint (x0, y1, z1);

  AddSegByPoint (0, 1);
  AddSegByPoint (1, 2);
  AddSegByPoint (2, 3);
  AddSegByPoint (3, 0);

  AddSegByPoint (4, 5);
  AddSegByPoint (5, 6);
  AddSegByPoint (6, 7);
  AddSegByPoint (7, 4);

  AddSegByPoint (0, 4);
  AddSegByPoint (1, 5);
  AddSegByPoint (2, 6);
  AddSegByPoint (3, 7);

  AddTriByPoint (0, 1, 5);
  AddTriByPoint (5, 4, 0);

  AddTriByPoint (6, 2, 3);
  AddTriByPoint (3, 7, 6);

  AddTriByPoint (2, 1, 0);
  AddTriByPoint (0, 3, 2);

  AddTriByPoint (4, 5, 6);
  AddTriByPoint (6, 7, 4);

  AddTriByPoint (0, 4, 7);
  AddTriByPoint (7, 3, 0);

  AddTriByPoint (1, 2, 6);
  AddTriByPoint (6, 5, 1);
}



int main (int argc, char *argv[])
{
  double xSize, ySize, zSize;

  if (argc != 7) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " gm3d_gm_box  x y z "
       "   point_file_OUT  seg_file_OUT  tri_file_OUT \n");
    exit (1);
  }

  xSize = atof (argv[1]);
  ySize = atof (argv[2]);
  zSize = atof (argv[3]);
  PerformCommand (xSize, ySize, zSize);

  WritePointFile (argv[4]);
  WriteSegFile (argv[5]);
  WriteTriFile (argv[6]);

  return 0;
}
