/* autoage_base.c : Base Module */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "autoage_base.h"



/* Utility */

void AutoAge_Error (const char *msg)
{
  fprintf (stderr, " AutoAge ERROR : %s\n", msg);
  abort ();
}

void AutoAge_Warning (const char *msg)
{
  fprintf (stderr, " AutoAge WARNING : %s\n", msg);
}



/* Floating Point Operations */

double AutoAGE_eps = 0.0000001;

int /* boolean */ 
AutoAGE_eq (double value0, double value1) 
{
  return value0 - AutoAGE_eps < value1 
    && value1 < value0 + AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_ne (double value0, double value1)
{
  return value1 < value0 - AutoAGE_eps 
    || value0 + AutoAGE_eps < value1;
}

int /* boolean */ 
AutoAGE_ge (double value0, double value1)
{
  return value0 > value1 - AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_le (double value0, double value1)
{
  return value0 < value1 + AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_gt (double value0, double value1)
{
  return value0 > value1 + AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_lt (double value0, double value1)
{
  return value0 < value1 - AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_between (double value, double value0, double value1)
{
  return (AutoAGE_le (value0, value) && AutoAGE_le (value, value1))
    || (AutoAGE_le (value1, value) && AutoAGE_le (value, value0));
}

int /* boolean */  
AutoAGE_justBetween (double value, double value0, double value1)
{
  return (AutoAGE_lt (value0, value) && AutoAGE_lt (value, value1))
    || (AutoAGE_lt (value1, value) && AutoAGE_lt (value, value0));
}

int /* boolean */ 
AutoAGE_eq2 (double value0, double value1) 
{
  return value0 - AutoAGE_eps * AutoAGE_eps < value1 
    && value1 < value0 + AutoAGE_eps * AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_ne2 (double value0, double value1)
{
  return value1 < value0 - AutoAGE_eps * AutoAGE_eps 
    || value0 + AutoAGE_eps * AutoAGE_eps < value1;
}

int /* boolean */ 
AutoAGE_ge2 (double value0, double value1)
{
  return value0 > value1 - AutoAGE_eps * AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_le2 (double value0, double value1)
{
  return value0 < value1 + AutoAGE_eps * AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_gt2 (double value0, double value1)
{
  return value0 > value1 + AutoAGE_eps * AutoAGE_eps;
}

int /* boolean */ 
AutoAGE_lt2 (double value0, double value1)
{
  return value0 < value1 - AutoAGE_eps * AutoAGE_eps;
}

int /* boolean */  
AutoAGE_between2 (double value, double value0, double value1)
{
  return (AutoAGE_le2 (value0, value) && AutoAGE_le2 (value, value1))
    || (AutoAGE_le2 (value1, value) && AutoAGE_le2 (value, value0));
}

int /* boolean */  
AutoAGE_justBetween2 (double value, double value0, double value1)
{
  return (AutoAGE_lt2 (value0, value) && AutoAGE_lt2 (value, value1))
    || (AutoAGE_lt2 (value1, value) && AutoAGE_lt2 (value, value0));
}

double AutoAGE_min (double value0, double value1) 
{
  return (value0 < value1) ? value0 : value1;
}

double AutoAGE_max (double value0, double value1) 
{
  return (value0 < value1) ? value1 : value0;
}

double AutoAGE_interpolate (double value0, double value1, double ratio) 
{
  return value0 + (value1 - value0) * ratio;
}

double AutoAGE_rangedValue (double value, double value0, double value1) 
{
  if (value0 < value1) {
    if (value < value0) {
      return value0;
    } else if (value1 < value) {
      return value1;
    } else {
      return value;
    }
  } else {
    if (value < value1) {
      return value1;
    } else if (value0 < value) {
      return value0;
    } else {
      return value;
    }
  }
}



