/* autoage_scalar.c : Scalar Operations */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "autoage_base.h"
#include "autoage_scalar.h"



/* Scalar */

void AutoAGE_Scalar_SortPair 
(double *value0_IO, double *value1_IO)
{
  if (*value1_IO < *value0_IO) {
    double dummy = *value0_IO;

    *value0_IO = *value1_IO;
    *value1_IO = dummy;
  }
}

int /* boolean */ 
AutoAGE_Scalar_MinMaxPairsOverlap 
(double min0, double max0, 
 double min1, double max1)
{
  /* PRE-CONDITION */
  assert(AutoAGE_le (min0, max0));
  assert(AutoAGE_le (min1, max1));
  
  if (AutoAGE_lt (max1, min0) || AutoAGE_lt (max0, min1)) {
    return 0;
  } else {
    return 1;
  }
}

int /* boolean */ 
AutoAGE_Scalar_ValuesOverlap 
(double value0_0, double value0_1, 
 double value1_0, double value1_1)
{
  AutoAGE_Scalar_SortPair (&value0_0, &value0_1);
  AutoAGE_Scalar_SortPair (&value1_0, &value1_1);
  return AutoAGE_Scalar_MinMaxPairsOverlap 
    (value0_0, value0_1, value1_0, value1_1);
}

AutoAGE_Scalar_OverlapType 
AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
(double min0, double max0, 
 double min1, double max1)
{
  /* PRE-CONDITION */
  assert(AutoAGE_lt (min0, max0));
  assert(AutoAGE_lt (min1, max1));

  if (AutoAGE_lt (max0, min1)) {
    return AUTOAGE_SCALAR_OVERLAP_HAS_NO_OVERLAP_MIN0_MAX0_MIN1_MAX1;
  } else if (AutoAGE_lt (max1, min0)) {
    return AUTOAGE_SCALAR_OVERLAP_HAS_NO_OVERLAP_MIN1_MAX1_MIN0_MAX0;
  } else if (AutoAGE_eq (max0, min1)) {
    return AUTOAGE_SCALAR_OVERLAP_TOUCHES_MIN0_MAX0_AS_MIN1_MAX1;
  } else if (AutoAGE_eq (max1, min0)) {
    return AUTOAGE_SCALAR_OVERLAP_TOUCHES_MIN1_MAX1_AS_MIN0_MAX0;
  } else if (AutoAGE_eq (min0, min1) 
	     && AutoAGE_eq (max0, max1)) {
    return AUTOAGE_SCALAR_OVERLAP_MATCHES_MIN0_AS_MIN1_MAX0_AS_MAX1;
  } else if (AutoAGE_lt (min0, min1) 
	     && AutoAGE_lt (min1, max0) 
	     && AutoAGE_lt (max0, max1)) {
    return AUTOAGE_SCALAR_OVERLAP_INTERSECTS_MIN0_MIN1_MAX0_MAX1;
  } else if (AutoAGE_lt (min1, min0) 
	     && AutoAGE_lt (min0, max1) 
	     && AutoAGE_lt (max1, max0)) {
    return AUTOAGE_SCALAR_OVERLAP_INTERSECTS_MIN1_MIN0_MAX1_MAX0;
  } else if (AutoAGE_lt (min0, min1) 
	     && AutoAGE_lt (max1, max0)) {
    return AUTOAGE_SCALAR_OVERLAP_CONTAINS_MIN0_MIN1_MAX1_MAX0;
  } else if (AutoAGE_lt (min1, min0) 
	     && AutoAGE_lt (max0, max1)) {
    return AUTOAGE_SCALAR_OVERLAP_IS_CONTAINED_MIN1_MIN0_MAX0_MAX1;
  } else if (AutoAGE_eq (min0, min1) 
	     && AutoAGE_lt (min1, max1) 
	     && AutoAGE_lt (max1, max0)) {
    return AUTOAGE_SCALAR_OVERLAP_CONTAINS_MIN0_AS_MIN1_MAX1_MAX0;
  } else if (AutoAGE_lt (min0, min1) 
	     && AutoAGE_lt (min1, max1) 
	     && AutoAGE_eq (max1, max0)) {
    return AUTOAGE_SCALAR_OVERLAP_CONTAINS_MIN0_MIN1_MAX1_AS_MAX0;
  } else if (AutoAGE_eq (min1, min0) 
	     && AutoAGE_lt (min0, max0) 
	     && AutoAGE_lt (max0, max1)) {
    return AUTOAGE_SCALAR_OVERLAP_IS_CONTAINED_MIN1_AS_MIN0_MAX0_MAX1;
  } else if (AutoAGE_lt (min1, min0) 
	     && AutoAGE_lt (min0, max0) 
	     && AutoAGE_eq (max0, max1)) {
    return AUTOAGE_SCALAR_OVERLAP_IS_CONTAINED_MIN1_MIN0_MAX0_AS_MAX1;
  } else {
    assert(0);
    return AUTOAGE_SCALAR_OVERLAP_HAS_NO_OVERLAP_MIN0_MAX0_MIN1_MAX1;
  }
}

double AutoAGE_Scalar_SolutionOfFirstOrderEquation 
(double a, double b)
{
  /* PRE-CONDITION */
  assert(AutoAGE_ne (a, 0));
  
  /* a x + b = 0 */
  
  return -b / a;
}

double AutoAGE_Scalar_Determinant2D 
(double a00, double a01,
 double a10, double a11)
{
  /* | a00   a01 | */
  /* | a10   a11 | */

  return a00 * a11 - a01 * a10;
}

double AutoAGE_Scalar_Determinant3D 
(double a00, double a01, double a02,
 double a10, double a11, double a12,
 double a20, double a21, double a22)
{
  /* | a00   a01   a02 | */
  /* | a10   a11   a12 | */
  /* | a20   a21   a22 | */

  return a00 * (a11 * a22 - a21 * a12) 
    - a01 * (a10 * a22 - a20 * a12)
      + a02 * (a10 * a21 - a20 * a11);
}

AutoAGE_Scalar_SecondOrderEquationSolutionType 
AutoAGE_Scalar_SolveSecondOrderEquation 
(double a, double b, double c,
 double *x0_OUT, double *x1_OUT)
{
  AutoAGE_Scalar_SecondOrderEquationSolutionType result;
  double b2_4ac;

  /* PRE-CONDITION */
  assert(AutoAGE_ne (a, 0));
  
  /* a x^2 + b x + c = 0 */
  
  result = AUTOAGE_SCALAR_SOES_IMAGINARY;

  b2_4ac = b * b - 4 * a * c;
  if (AutoAGE_eq2 (b2_4ac, 0)) {
    *x0_OUT = *x1_OUT = -b / (2 * a);

    result = AUTOAGE_SCALAR_SOES_DUPLICATED_REAL;
  }

  if (result != AUTOAGE_SCALAR_SOES_DUPLICATED_REAL) {
    if (0 < b2_4ac) {
      if (0 < a) {
	*x0_OUT = (-b - sqrt (b2_4ac)) / (2 * a);
	*x1_OUT = (-b + sqrt (b2_4ac)) / (2 * a);
      } else {
	*x0_OUT = (-b + sqrt (b2_4ac)) / (2 * a);
	*x1_OUT = (-b - sqrt (b2_4ac)) / (2 * a);
      }

      result = AUTOAGE_SCALAR_SOES_REAL;
    }
  }
  
  return result;
}

int /* boolean */
AutoAGE_Scalar_SolveLinearEquation2D 
(double a00, double a01,
 double a10, double a11, 
 double b0, double b1,
 double *x_OUT, double *y_OUT)
{
  int result = 0;
  double det;

  /* a00 x + a01 y + b0 = 0 */
  /* a10 x + a11 y + b1 = 0 */

  det = AutoAGE_Scalar_Determinant2D 
    (a00, a01, 
     a10, a11);
  if (AutoAGE_ne2 (det, 0)) {
    *x_OUT = (a01 * b1 - a11 * b0) / det;
    *y_OUT = (a10 * b0 - a00 * b1) / det;

    result = 1;
  }

  return result;
}

int AutoAGE_Scalar_SolveLinearEquation3D 
(double a00, double a01, double a02,
 double a10, double a11, double a12, 
 double a20, double a21, double a22, 
 double b0, double b1, double b2,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  int result = 0;
  double det;

  /* a00 x + a01 y + a02 z + b0 = 0 */
  /* a10 x + a11 y + a12 z + b1 = 0 */
  /* a20 x + a21 y + a22 z + b2 = 0 */

  det = AutoAGE_Scalar_Determinant3D 
    (a00, a01, a02, 
     a10, a11, a12,
     a20, a21, a22);
  if (AutoAGE_ne2 (det, 0)) {
    double bc = a11 * a22 - a21 * a12;
    double ac = a10 * a22 - a20 * a12;
    double ab = a10 * a21 - a20 * a11;
    double dc = b1 * a22 - b2 * a12;
    double db = b1 * a21 - b2 * a11;
    double ad = a10 * b2 - a20 * b1;

    *x_OUT = (a01 * dc - b0 * bc - a02 * db) / det;
    *y_OUT = (b0 * ac - a00 * dc - a02 * ad) / det;
    *z_OUT = (a01 * ad + a00 * db - b0 * ab) / det;

    result = 1;
  }

  return result;
}



