/* autoage_vector2d.c : 2-D Vector Operations */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "autoage_base.h"
#include "autoage_scalar.h"
#include "autoage_vector2d.h"



/* Vector2D */

AutoAGE_Vector2D::AutoAGE_Vector2D () 
{
  this->x = 0.0;  this->y = 0.0; 
}

AutoAGE_Vector2D::AutoAGE_Vector2D (double x, double y) 
{ 
  this->x = x;  this->y = y; 
}

AutoAGE_Vector2D AutoAGE_Vector2D::operator- () 
{ 
  return AutoAGE_Vector2D (-x, -y); 
}

AutoAGE_Vector2D& AutoAGE_Vector2D::operator+= (const AutoAGE_Vector2D& p) 
{ 
  x += p.x;  y += p.y;  return *this; 
}

AutoAGE_Vector2D& AutoAGE_Vector2D::operator-= (const AutoAGE_Vector2D& p) 
{ 
  x -= p.x;  y -= p.y;  return *this; 
}

AutoAGE_Vector2D& AutoAGE_Vector2D::operator*= (double ratio) 
{ 
  x *= ratio;  y *= ratio;  return *this; 
}

AutoAGE_Vector2D& AutoAGE_Vector2D::operator/= (double ratio) 
{ 
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_ne (ratio, 0.0));

  x /= ratio;  y /= ratio;  return *this; 
}

double AutoAGE_Vector2D::length ()
{ 
  return sqrt (x * x + y * y); 
}

AutoAGE_Vector2D AutoAGE_Vector2D::normalize () 
{
  double len = this->length ();
  AutoAGE_ASSERT(AutoAGE_ne (len, 0.0));
  return AutoAGE_Vector2D (x / len, y / len);
}

AutoAGE_Vector2D AutoAGE_abs (const AutoAGE_Vector2D& p)
{ 
  return AutoAGE_Vector2D (p.x > 0 ? p.x : -p.x, p.y > 0 ? p.y : -p.y); 
}

AutoAGE_Vector2D operator+ 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_Vector2D (p0.x + p1.x, p0.y + p1.y);
}

AutoAGE_Vector2D operator- 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_Vector2D (p0.x - p1.x, p0.y - p1.y);
}

AutoAGE_Vector2D operator* 
(const AutoAGE_Vector2D& p0, double ratio) 
{
  return AutoAGE_Vector2D (p0.x * ratio, p0.y * ratio);
}

AutoAGE_Vector2D operator/ 
(const AutoAGE_Vector2D& p0, double ratio) 
{
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_ne (ratio, 0.0));

  return AutoAGE_Vector2D (p0.x / ratio, p0.y / ratio);
}

double AutoAGE_scalarProduct 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return p0.x * p1.x + p0.y * p1.y;
}

AutoAGE_Vector2D AutoAGE_multiply 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_Vector2D (p0.x * p1.x, p0.y * p1.y);
}

AutoAGE_Vector2D AutoAGE_divide 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_ne (p1.x, 0.0));
  AutoAGE_ASSERT(AutoAGE_ne (p1.y, 0.0));

  return AutoAGE_Vector2D (p0.x / p1.x, p0.y / p1.y);
}

bool AutoAGE_eq 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_eq (p0.x, p1.x) && AutoAGE_eq (p0.y, p1.y);
}

bool AutoAGE_ne 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_ne (p0.x, p1.x) || AutoAGE_ne (p0.y, p1.y);
}

bool operator== 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1)
{
  return AutoAGE_eq (p0, p1);
}

bool operator!= 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1)
{
  return AutoAGE_ne (p0, p1);
}

bool AutoAGE_lt
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_lt (p0.x, p1.x) && AutoAGE_lt (p0.y, p1.y);
}

bool AutoAGE_gt
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_gt (p0.x, p1.x) && AutoAGE_gt (p0.y, p1.y);
}

bool AutoAGE_le
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_le (p0.x, p1.x) && AutoAGE_le (p0.y, p1.y);
}

bool AutoAGE_ge
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_ge (p0.x, p1.x) && AutoAGE_ge (p0.y, p1.y);
}

bool AutoAGE_eq2 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_eq2 (p0.x, p1.x) && AutoAGE_eq2 (p0.y, p1.y);
}

bool AutoAGE_ne2 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_ne2 (p0.x, p1.x) || AutoAGE_ne2 (p0.y, p1.y);
}

bool AutoAGE_lt2
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_lt2 (p0.x, p1.x) && AutoAGE_lt2 (p0.y, p1.y);
}

bool AutoAGE_gt2
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_gt2 (p0.x, p1.x) && AutoAGE_gt2 (p0.y, p1.y);
}

bool AutoAGE_le2
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_le2 (p0.x, p1.x) && AutoAGE_le2 (p0.y, p1.y);
}

bool AutoAGE_ge2
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_ge2 (p0.x, p1.x) && AutoAGE_ge2 (p0.y, p1.y);
}

AutoAGE_Vector2D AutoAGE_min 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_Vector2D (p0.x < p1.x ? p0.x : p1.x,
		      p0.y < p1.y ? p0.y : p1.y);
}

AutoAGE_Vector2D AutoAGE_max 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_Vector2D (p0.x > p1.x ? p0.x : p1.x,
		      p0.y > p1.y ? p0.y : p1.y);
}

bool AutoAGE_isReverse 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return AutoAGE_eq (p0.x, -p1.x) && AutoAGE_eq (p0.y, -p1.y);
}

bool AutoAGE_isParallel 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1)
{
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_ne (p0, AutoAGE_Vector2D::zero ()));
  AutoAGE_ASSERT(AutoAGE_ne (p1, AutoAGE_Vector2D::zero ()));

  double c = AutoAGE_scalarProduct (p0, p1) / (p0.length () * p1.length ());
  return AutoAGE_eq (fabs (c), 1);
}

bool AutoAGE_isPerpendicular 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1)
{
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_ne (p0, AutoAGE_Vector2D::zero ()));
  AutoAGE_ASSERT(AutoAGE_ne (p1, AutoAGE_Vector2D::zero ()));

  return AutoAGE_eq (AutoAGE_scalarProduct (p0, p1), 0);
}

double AutoAGE_distance 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  return (p0 - p1).length ();
}

double AutoAGE_squareSum 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  double x = p0.x - p1.x;
  double y = p0.y - p1.y;
  return x * x + y * y;
}

void AutoAGE_swap 
(AutoAGE_Vector2D& p0, AutoAGE_Vector2D& p1) 
{
  AutoAGE_Vector2D dummy = p0;  
  p0 = p1; 
  p1 = dummy;
}

AutoAGE_Vector2D AutoAGE_interpolate 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1, double ratio) 
{
  return p0 + (p1 - p0) * ratio;
}

void AutoAGE_sortPair 
(AutoAGE_Vector2D& value0_IO, AutoAGE_Vector2D& value1_IO)
{
  AutoAGE_sortPair (value0_IO.x, value1_IO.x);
  AutoAGE_sortPair (value0_IO.y, value1_IO.y);
}

bool AutoAGE_minMaxPairsOverlap 
(const AutoAGE_Vector2D& min0, const AutoAGE_Vector2D& max0, 
 const AutoAGE_Vector2D& min1, const AutoAGE_Vector2D& max1)
{
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_le (min0, max0));
  AutoAGE_ASSERT(AutoAGE_le (min1, max1));

  return AutoAGE_minMaxPairsOverlap (min0.x, max0.x, min1.x, max1.x)
    && AutoAGE_minMaxPairsOverlap (min0.y, max0.y, min1.y, max1.y);
}

double AutoAGE_signedTriangleAreaAmong 
(const AutoAGE_Vector2D& p0, 
 const AutoAGE_Vector2D& p1, 
 const AutoAGE_Vector2D& p2)
{
  /* p0 ... K */
  /* p1 ... L */
  /* p2 ... M */
  
  AutoAGE_Vector2D lk = p1 - p0;
  AutoAGE_Vector2D mk = p2 - p0;
  return (lk.x * mk.y - mk.x * lk.y) / 2;
}

AutoAGE_Vector2D AutoAGE_Vector2D::zero ()
{
  return AutoAGE_Vector2D (0, 0);
}

AutoAGE_Vector2D AutoAGE_Vector2D::xUnit ()
{
  return AutoAGE_Vector2D (1, 0);
}

AutoAGE_Vector2D AutoAGE_Vector2D::yUnit ()
{
  return AutoAGE_Vector2D (0, 1);
}

AutoAGE_Vector2D AutoAGE_Vector2D::hugeValue ()
{
  return AutoAGE_Vector2D (1.0e+20, 1.0e+20);
}

ostream& operator<< (ostream& out, const AutoAGE_Vector2D& p) 
{
  out << "( " << AutoAGE_format (p.x) << " " << AutoAGE_format (p.y) << " ) ";
  return out;
}

bool AutoAGE_between (const AutoAGE_Vector2D& p, 
		 const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  double x = p.x;
  double y = p.y;
  if (p0.x <= p1.x) {
    if (AutoAGE_lt (x, p0.x) || AutoAGE_lt (p1.x, x)) return false;
  } else {
    if (AutoAGE_lt (x, p1.x) || AutoAGE_lt (p0.x, x)) return false;
  }
  if (p0.y <= p1.y) {
    if (AutoAGE_lt (y, p0.y) || AutoAGE_lt (p1.y, y)) return false;
  } else {
    if (AutoAGE_lt (y, p1.y) || AutoAGE_lt (p0.y, y)) return false;
  }
  return true;
}

bool AutoAGE_justBetween (const AutoAGE_Vector2D& p, 
		     const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  double x = p.x;
  double y = p.y;
  if (p0.x <= p1.x) {
    if (AutoAGE_le (x, p0.x) || AutoAGE_le (p1.x, x)) return false;
  } else {
    if (AutoAGE_le (x, p1.x) || AutoAGE_le (p0.x, x)) return false;
  }
  if (p0.y <= p1.y) {
    if (AutoAGE_le (y, p0.y) || AutoAGE_le (p1.y, y)) return false;
  } else {
    if (AutoAGE_le (y, p1.y) || AutoAGE_le (p0.y, y)) return false;
  }
  return true;
}

bool AutoAGE_between2 (const AutoAGE_Vector2D& p, 
		  const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  double x = p.x;
  double y = p.y;
  if (p0.x <= p1.x) {
    if (AutoAGE_lt2 (x, p0.x) || AutoAGE_lt2 (p1.x, x)) return false;
  } else {
    if (AutoAGE_lt2 (x, p1.x) || AutoAGE_lt2 (p0.x, x)) return false;
  }
  if (p0.y <= p1.y) {
    if (AutoAGE_lt2 (y, p0.y) || AutoAGE_lt2 (p1.y, y)) return false;
  } else {
    if (AutoAGE_lt2 (y, p1.y) || AutoAGE_lt2 (p0.y, y)) return false;
  }
  return true;
}

bool AutoAGE_justBetween2 (const AutoAGE_Vector2D& p, 
		      const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1) 
{
  double x = p.x;
  double y = p.y;
  if (p0.x <= p1.x) {
    if (AutoAGE_le2 (x, p0.x) || AutoAGE_le2 (p1.x, x)) return false;
  } else {
    if (AutoAGE_le2 (x, p1.x) || AutoAGE_le2 (p0.x, x)) return false;
  }
  if (p0.y <= p1.y) {
    if (AutoAGE_le2 (y, p0.y) || AutoAGE_le2 (p1.y, y)) return false;
  } else {
    if (AutoAGE_le2 (y, p1.y) || AutoAGE_le2 (p0.y, y)) return false;
  }
  return true;
}

AutoAGE_Vector2D AutoAGE_Vector2D::translate (const AutoAGE_Vector2D& offset) 
{
  return *this + offset;
}

AutoAGE_Vector2D AutoAGE_Vector2D::rotate (const AutoAGE_Vector2D& center, double angle) 
{
  AutoAGE_CVR_LINE;

  AutoAGE_Vector2D v = *this - center;
  double s = sin (angle);
  double c = cos (angle);
  v = AutoAGE_Vector2D (c * v.x - s * v.y, s * v.x + c * v.y);
  return center + v;
}

AutoAGE_Vector2D AutoAGE_Vector2D::scale (const AutoAGE_Vector2D& center, double ratio) 
{
  return center + (*this - center) * ratio;
}

double AutoAGE_Vector2D ::slope ()
{
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_ne (x, 0) || AutoAGE_ne (y, 0));

  double result = M_PI * 2;

  if (AutoAGE_eq (x, 0)) {
    result = (y > 0) ? M_PI / 2 : -M_PI / 2;
  } else if (AutoAGE_eq (y, 0)) {
    result = (x > 0) ? 0 : M_PI;
  } else {
    if (0 < x) {
      if (0 < y) {
	result = atan (y / x);
      } else {
	result = -atan (-y / x);
      }
    } else {
      if (0 < y) {
	result = M_PI - atan (y / -x);
      } else {
	result = -M_PI + atan (-y / -x);
      }
    }
  }

  return result;
}

double AutoAGE_angleOfVectors 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1)
{
  double slope0 = p0.slope ();
  double slope1 = p1.slope ();
  double result = slope1 - slope0;
  if (AutoAGE_lt (M_PI, result)) {
    result -= 2 * M_PI;
  } else if (AutoAGE_le (result, -M_PI)) {
    result += 2 * M_PI;
  }

  return result;
}

bool AutoAGE_getScaleIfParallel 
(const AutoAGE_Vector2D& p0, const AutoAGE_Vector2D& p1,
 double& ratio_OUT)
{
  /* PRE-CONDITION */
  AutoAGE_ASSERT(AutoAGE_ne (p0, AutoAGE_Vector2D::zero ()));
  AutoAGE_ASSERT(AutoAGE_ne (p1, AutoAGE_Vector2D::zero ()));

  bool result = false;
  if (AutoAGE_isParallel (p0, p1)) {
    result = true;
    if (AutoAGE_ne (p1.x, 0)) {
      ratio_OUT = p0.x / p1.x;
    } else {
      AutoAGE_ASSERT(AutoAGE_ne (p1.y, 0));
      ratio_OUT = p0.y / p1.y;
    }
  }
  
  return result;
}

bool AutoAGE_valuesOverlap 
(const AutoAGE_Vector2D& value0_0, const AutoAGE_Vector2D& value0_1, 
 const AutoAGE_Vector2D& value1_0, const AutoAGE_Vector2D& value1_1)
{
  AutoAGE_Vector2D newValue0_0 = value0_0;
  AutoAGE_Vector2D newValue0_1 = value0_1;
  AutoAGE_sortPair (newValue0_0, newValue0_1);
  AutoAGE_Vector2D newValue1_0 = value1_0;
  AutoAGE_Vector2D newValue1_1 = value1_1;
  AutoAGE_sortPair (newValue1_0, newValue1_1);
  return AutoAGE_minMaxPairsOverlap 
    (newValue0_0, newValue0_1, newValue1_0, newValue1_1);
}

AutoAGE_Vector2D AutoAGE_circumcenterAmong 
(const AutoAGE_Vector2D& p0, 
 const AutoAGE_Vector2D& p1, 
 const AutoAGE_Vector2D& p2)
{
  double x0 = p0.x;
  double y0 = p0.y;
  double x1 = p1.x;
  double y1 = p1.y;
  double x2 = p2.x;
  double y2 = p2.y;
  double a = ((x0 * x0 - x1 * x1 + y0 * y0 - y1 * y1) * (y0 - y2)
	      - (x0 * x0 - x2 * x2 + y0 * y0 - y2 * y2) * (y0 - y1))
    / ((x0 - x1) * (y0 - y2) - (x0 - x2) * (y0 - y1));
  a = a / 2;
  double b = ((x0 * x0 - x1 * x1 + y0 * y0 - y1 * y1) * (x0 - x2)
	      - (x0 * x0 - x2 * x2 + y0 * y0 - y2 * y2) * (x0 - x1))
    / ((y0 - y1) * (x0 - x2) - (y0 - y2) * (x0 - x1));
  b = b / 2;
  return AutoAGE_Vector2D (a, b);
}



