/* test_scalar.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "autoage_base.h"
#include "autoage_scalar.h"



static void Test0 (void)
{



  printf (" Test0 \n"); 
  
  
  
  assert(AutoAGE_eq 
	 (AutoAGE_Scalar_SolutionOfFirstOrderEquation (2, -4), 2));
  
  {
    AutoAGE_Scalar_SecondOrderEquationSolutionType flag;
    double x0;
    double x1;
    
    flag = AutoAGE_Scalar_SolveSecondOrderEquation 
      (2, -5, 2, 
       &x0, &x1);
    assert(flag == AUTOAGE_SCALAR_SOES_REAL);
    assert(AutoAGE_eq (x0, 0.5));
    assert(AutoAGE_eq (x1, 2));

    flag = AutoAGE_Scalar_SolveSecondOrderEquation 
      (1, 0, -4, 
       &x0, &x1);
    assert(flag == AUTOAGE_SCALAR_SOES_REAL);
    assert(AutoAGE_eq (x0, -2));
    assert(AutoAGE_eq (x1, 2));

    flag = AutoAGE_Scalar_SolveSecondOrderEquation 
      (-1, 0, 4, 
       &x0, &x1);
    assert(flag == AUTOAGE_SCALAR_SOES_REAL);
    assert(AutoAGE_eq (x0, -2));
    assert(AutoAGE_eq (x1, 2));
    
    flag = AutoAGE_Scalar_SolveSecondOrderEquation 
      (1, 2, 1, 
       &x0, &x1);
    assert(flag == AUTOAGE_SCALAR_SOES_DUPLICATED_REAL);
    assert(AutoAGE_eq (x0, -1));
    assert(AutoAGE_eq (x1, -1));
    
    flag = AutoAGE_Scalar_SolveSecondOrderEquation 
      (2, 1, 1, 
       &x0, &x1);
    assert(flag == AUTOAGE_SCALAR_SOES_IMAGINARY);
  }
  
  {
    double x;
    double y;
    int flag;

    assert(AutoAGE_eq 
	   (AutoAGE_Scalar_Determinant2D (1, 2, 
					  4, 5), -3));
    
    flag = AutoAGE_Scalar_SolveLinearEquation2D 
      (1, 2, 
       4, 5,
       3, 6,
       &x, &y);
    assert(flag);
    assert(AutoAGE_eq (x, 1));
    assert(AutoAGE_eq (y, -2));

    flag = AutoAGE_Scalar_SolveLinearEquation2D 
      (1, 2, 
       4, 8,
       3, 6,
       &x, &y);
    assert(!flag);
  }

  {
    double x;
    double y;
    double z;
    int flag;

    assert(AutoAGE_eq 
	   (AutoAGE_Scalar_Determinant3D (1, 0, 0,
					  0, 1, 0,
					  0, 0, 1), 1));
    
    flag = AutoAGE_Scalar_SolveLinearEquation3D 
      (1, 0, 0,
       0, 1, 0,
       0, 0, 1,
       1, 2, 3,
       &x, &y, &z);
    assert(flag);
    assert(AutoAGE_eq (x, -1));
    assert(AutoAGE_eq (y, -2));
    assert(AutoAGE_eq (z, -3));

    flag = AutoAGE_Scalar_SolveLinearEquation3D 
      (1, 2, 3,
       4, 5, 3,
       3, 2, 1,
       -14, -23, -10,
       &x, &y, &z);
    assert(flag);
    assert(AutoAGE_eq (x, 1));
    assert(AutoAGE_eq (y, 2));
    assert(AutoAGE_eq (z, 3));

  }

  {
    double value0 = 1.0;
    double value1 = 2.0;

    AutoAGE_Scalar_SortPair (&value0, &value1);
    assert(AutoAGE_lt (value0, value1));
    assert(AutoAGE_eq (value0, 1.0));
    assert(AutoAGE_eq (value1, 2.0));
    
    AutoAGE_Scalar_SortPair (&value1, &value0);
    assert(AutoAGE_lt (value1, value0));
    assert(AutoAGE_eq (value1, 1.0));
    assert(AutoAGE_eq (value0, 2.0));
  }

  {
    assert(!AutoAGE_Scalar_ValuesOverlap 
	   (2, 1, 4, 3));
    assert(!AutoAGE_Scalar_MinMaxPairsOverlap 
	   (1, 2, 3, 4));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (1, 2, 3, 4) 
	   == AUTOAGE_SCALAR_OVERLAP_HAS_NO_OVERLAP_MIN0_MAX0_MIN1_MAX1);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (2, 1, 3, 2));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (1, 2, 2, 3));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (1, 2, 2, 3) 
	   == AUTOAGE_SCALAR_OVERLAP_TOUCHES_MIN0_MAX0_AS_MIN1_MAX1);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (3, 1, 4, 2));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap
	   (1, 3, 2, 4));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (1, 3, 2, 4) 
	   == AUTOAGE_SCALAR_OVERLAP_INTERSECTS_MIN0_MIN1_MAX0_MAX1);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (4, 1, 3, 2));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (1, 4, 2, 3));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (1, 4, 2, 3) 
	   == AUTOAGE_SCALAR_OVERLAP_CONTAINS_MIN0_MIN1_MAX1_MAX0);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (4, 1, 3, 1));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (1, 4, 1, 3));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (1, 4, 1, 3) 
	   == AUTOAGE_SCALAR_OVERLAP_CONTAINS_MIN0_AS_MIN1_MAX1_MAX0);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (3, 1, 3, 2));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (1, 3, 2, 3));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (1, 3, 2, 3) 
	   == AUTOAGE_SCALAR_OVERLAP_CONTAINS_MIN0_MIN1_MAX1_AS_MAX0);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (3, 1, 3, 1));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (1, 3, 1, 3));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (1, 3, 1, 3) 
	   == AUTOAGE_SCALAR_OVERLAP_MATCHES_MIN0_AS_MIN1_MAX0_AS_MAX1);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (3, 2, 4, 1));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 3, 1, 4));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (2, 3, 1, 4) 
	   == AUTOAGE_SCALAR_OVERLAP_IS_CONTAINED_MIN1_MIN0_MAX0_MAX1);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (3, 2, 4, 2));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 3, 2, 4));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (2, 3, 2, 4) 
	   == AUTOAGE_SCALAR_OVERLAP_IS_CONTAINED_MIN1_AS_MIN0_MAX0_MAX1);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (3, 2, 2, 1));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 3, 1, 3));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (2, 3, 1, 3) 
	   == AUTOAGE_SCALAR_OVERLAP_IS_CONTAINED_MIN1_MIN0_MAX0_AS_MAX1);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (4, 2, 3, 1));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 4, 1, 3));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (2, 4, 1, 3) 
	   == AUTOAGE_SCALAR_OVERLAP_INTERSECTS_MIN1_MIN0_MAX1_MAX0);
    
    assert(AutoAGE_Scalar_ValuesOverlap 
	   (3, 2, 2, 1));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 3, 1, 2));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (2, 3, 1, 2) 
	   == AUTOAGE_SCALAR_OVERLAP_TOUCHES_MIN1_MAX1_AS_MIN0_MAX0);
    
    assert(!AutoAGE_Scalar_ValuesOverlap 
	   (4, 3, 2, 1));
    assert(!AutoAGE_Scalar_MinMaxPairsOverlap 
	   (3, 4, 1, 2));
    assert(AutoAGE_Scalar_CheckOverlapOfMinMaxPairs 
	   (3, 4, 1, 2) 
	   == AUTOAGE_SCALAR_OVERLAP_HAS_NO_OVERLAP_MIN1_MAX1_MIN0_MAX0);
    
    
    assert(!AutoAGE_Scalar_MinMaxPairsOverlap 
	   (3, 3, 1, 2));
    assert(!AutoAGE_Scalar_MinMaxPairsOverlap 
	   (3, 3, 2, 2));
    assert(!AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 3, 1, 1));
    
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 2, 1, 3));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (1, 1, 1, 3));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (3, 3, 3, 3));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 4, 3, 3));
    assert(AutoAGE_Scalar_MinMaxPairsOverlap 
	   (2, 4, 2, 2));
  }
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();

  return 0;
}


